!   !
!    Library "GRAPHICA"

!----------------------------------------
module GRAPHICA
implicit none

    contains

subroutine Graphic(A, B, Fun)
!declaration
interface
  real function Fun(X)
    real, intent(in):: X
  end function Fun
end interface
real, intent(in):: A, B
integer, parameter:: n=60
real:: F(n), Fmin, Fmax, StepX, StepF, SX, SF
character, parameter:: pr=' ', zv='*'
character:: L(n)
integer i, k
!begin
  StepX=(B-A)/float(n-1)
  do i=1, n
    F(i)=Fun(A+StepX*float(i-1))
  end do
  Fmin=minval(F); Fmax=maxval(F)
  StepF=(Fmax-Fmin)/float(n-1)
  open(1, file='gf.txt')
  write(1,40)
  do i=1, n
    SX=B-StepX*float(i-1)
    SF=Fmax-StepF*float(i-1)
    L(:)=pr
    do k=1, n
      if (abs(SF-F(k))<0.5*StepF) L(k)=zv
    end do
    write(1,50) i, SF, L, i, SX
  end do
  write(1,60)
  close(1)
  return
40 format(/42x,'  Fun(X)'    &
          /17x,' Y'/9x,'',73x,'')
50 format(2x,I3.1,E13.5,' |',60A1,I5.1,E13.5)
60 format(19x,61('-')/75x,' X')
end subroutine Graphic


!----------------------------------------
subroutine Graphar(X, Y)
!declaration
real, intent(in):: X(:), Y(:)
integer, allocatable:: IX(:), IY(:)
integer, parameter:: nr=50, nc=90
real:: Xmin, Xmax, Ymin, Ymax, SX, SY
character, parameter:: pr=' ', zv='*'
character:: Line(nc)
integer i, k, m, n, nt, nstr, L, R
!begin
  n=size(X)
  allocate(IX(n), IY(n))
  Xmin=minval(X); Xmax=maxval(X)
  Ymin=minval(Y); Ymax=maxval(Y)
  if (n==1) then
    IX(1)=nc/2; IY(1)=nr/2; goto 10
  end if
  SX=float(nc-1)/(Xmax-Xmin)
  SY=float(nr-1)/(Ymax-Ymin)
  do i=1, n
    IX(i)=int((X(i)-Xmin)*SX+1.5)
    IY(i)=int((Y(i)-Ymin)*SY+1.5)
  end do
  call QSort(1,n)
10 continue
  open(1, file='gr.txt')
  write(1,100) Xmin, Xmax, Ymin, Ymax
  nt=0
  do i=1, nr
    Line(:)=pr
    nstr=nr+1-i
20  nt=nt+1
    if (nt>n) goto 40
    m=IY(nt)
    if (m/=nstr) goto 30
    m=IX(nt); Line(m)=zv
    goto 20
30  nt=nt-1
40  write(1,110) Line
  end do
  write(1,120)
  close(1)
  deallocate(IX,IY)
  return
100 format(/40x,'  Y(X)'    &
  //10x,'Xmin =',E15.7,4x,'Xmax =',E15.7     &
   /10x,'Ymin =',E15.7,4x,'Ymax =',E15.7//2x,' Y')
110 format('    |',90A1,'.')
120 format(4x,91('-')/90x,' X')

contains

recursive subroutine QSort(L, R)
integer, intent(in):: L, R
integer L1, R1
integer W
!begin
  L1=L; R1=R
  if(L1==R1) return
  if(L1+1==R1) then
    if(IY(L1)<IY(R1)) then
      W=IX(L1); IX(L1)=IX(R1); IX(R1)=W
      W=IY(L1); IY(L1)=IY(R1); IY(R1)=W
    end if
    return
  else if(L1+2==R1) then
    if(IY(L1)<IY(L1+1)) then
      W=IX(L1); IX(L1)=IX(L1+1); IX(L1+1)=W
      W=IY(L1); IY(L1)=IY(L1+1); IY(L1+1)=W
    end if
    if(IY(L1+1)<IY(R1)) then
      W=IX(L1+1); IX(L1+1)=IX(R1); IX(R1)=W
      W=IY(L1+1); IY(L1+1)=IY(R1); IY(R1)=W
      if(IY(L1)<IY(L1+1)) then
        W=IX(L1); IX(L1)=IX(L1+1); IX(L1+1)=W
        W=IY(L1); IY(L1)=IY(L1+1); IY(L1+1)=W
      end if
    end if
    return
  end if
AA: DO
    do while(IY(L1)>=IY(R1))
      if(L1==R1) exit AA
      R1=R1-1
    end do
    W=IX(L1); IX(L1)=IX(R1); IX(R1)=W
    W=IY(L1); IY(L1)=IY(R1); IY(R1)=W
    L1=L1+1
    do while(IY(L1)>=IY(R1))
      if(L1==R1) exit AA
      L1=L1+1
    end do
    W=IX(L1); IX(L1)=IX(R1); IX(R1)=W
    W=IY(L1); IY(L1)=IY(R1); IY(R1)=W
    R1=R1-1
  END DO AA
  if(L<R1-1) call QSort(L, R1-1)
  if(R1+1<R) call QSort(R1+1, R)
  return
end subroutine QSort

end subroutine Graphar


!----------------------------------------
subroutine Graphop(Xmin, Xmax, Ymin, Ymax, Fun)
!declaration
interface
  real function Fun(x, y)
    real, intent(in):: x, y
  end function Fun
end interface
real, intent(in):: Xmin, Xmax, Ymin, Ymax
integer, parameter:: n=60, p=10
real:: F(n,n), FP(p), Fmin, Fmax, StepF, StepX, StepY, SX, SY
character, parameter:: pr=' '
character:: L(n), Z(p)
data Z/'0','1','2','3','4','5','6','7','8','9'/
integer i, j, k
!begin
  StepX=(Xmax-Xmin)/float(n-1)
  StepY=(Ymax-Ymin)/float(n-1)
  Fmin=Fun(Xmin, Ymin); Fmax=Fmin
  do j=1, n
    SY=Ymax-StepY*(j-1)
    do i=1, n
      SX=Xmin+StepX*(i-1)
      F(i,j)=Fun(SX, SY)
      if (Fmin>F(i,j)) Fmin=F(i,j)
      if (Fmax<F(i,j)) Fmax=F(i,j)
    end do
  end do
  StepF=(Fmax-Fmin)/float(p-1)
  do i=1, p
    FP(i)=Fmin+StepF*float(i-1)
  end do
  StepF=0.5*(Fmax-Fmin)/float(n-1)
  open(1, file='gp.txt')
  write(1,40) (Z(i), FP(i), i=1, p)
  do j=1, n
    SY=Ymax-StepY*(j-1)
    SX=Xmax-StepX*(j-1)
    L(:)=pr
    do i=1, n
      do k=1, p
        if (abs(F(i,j)-FP(k))<StepF) L(i)=Z(k)
      end do
    end do
    write(1,50) j, SY, L, j, SX
  end do
  write(1,60)
  close(1)
  return
40 format(/40x,'  Fun(x, y)'/       &
   10(/4x,',   ',A1,     &
   ',   , ',E15.7)  &
   //17x,' Y'/12x,'Y',80x,'X')
50 format(2x,I3.1,E13.5,' |',60A1,I5.1,E13.5)
60 format(19x,61('-')/75x,' X')
end subroutine Graphop

end module GRAPHICA